#!/usr/dist/bin/wishx -f

#
# appointment.tcl
# Copyright (C) 1995 by John Heidemann
# This program is released under the Gnu Public License, version 2.
# Comments to <johnh@isi.edu>.
#
# $Id: appointment.tcl,v 1.10 1996/12/31 00:25:22 johnh Exp $
#

proc max {a b} {
	if {$a > $b} {
		return $a
	} else {
		return $b
	}
}

proc appointment_destroy {tl} {
	if [winfo exists $tl] {
		destroy $tl
	}
}

proc appointment_positon_under {new old} {
	set geo [wm geometry $old]
	if [regexp {([0-9]+)x([0-9]+)\+([0-9]+)\+([0-9]+)} $geo t h w x y] {
		set x [expr $x+$h/2]
		set y [expr $y+$w/2]
		wm geometry $new "+$x+$y"
	} else {
		puts stderr "monitor: geometry for $old failed."
	}
}

proc appointment_puts {s} {
	# for debugging
	# puts $s
}

proc appointment_signal_appts {} {
	global appt_times appt_data appt_width appt_height appts_shown appointment rdb

	appointment_puts appointment_signal_appts
	set clock [getclock]
	appointment_puts "appointment: current time $clock, appt_times: $appt_times"

	set tl ".appt_$clock"

	# create a new popup box.
	toplevel $tl
	appointment_positon_under $tl .
	set title "Appt: [fmtclock $clock %I:%M]"
	wm iconname $tl $title
	wm title $tl $title
	wm transient $tl

	# Pick the text.
	set t {}
	set new_first_appt 999999999
	set h 0
	set w 0
	set num_shown 0
	foreach i $appt_times {
		if {$i <= $clock} {
			appointment_puts "trial: $i"
		}
		if {$i <= $clock && ![info exists appts_shown($i)]} {
			appointment_puts "showing appt $i: $appt_data($i)"
			set appts_shown($i) 1
			set w [max $tl $appt_width($i)]
			set h [expr $h+$appt_height($i)]
			set t "$t$appt_data($i)\n"
			incr num_shown
		}
		if {$i > $clock} {
			set new_first_appt $i
			break
		}
	}
	set appointment(first_appt) $new_first_appt
	appointment_puts "appts to show: $num_shown"
	if {$num_shown == 0} {
		destroy $tl
		return
	}

	# fill it with anything interesting.
	text $tl.t -wrap word -font $rdb(font) \
		-width $w \
		-height $h
#		-yscrollcommand "$tl.s set"
	$tl.t insert 0.0 $t
	$tl.t conf -state disabled
	# scrollbar
#	scrollbar $tl.s -relief flat -command "$tl.t yview"
# 	pack $tl.s {right filly} $tl.t {expand fill}
	pack $tl.t

	# Make an "ok button"
	button $tl.b -text "Ok" -command "appointment_destroy $tl"
	pack $tl.b -side right -ipadx $rdb(appointment.padx) -padx $rdb(appointment.padx)
	# NEEDSWORK.

	# Auto-pop-down the box after one hour.
	after 3600000 "appointment_destroy $tl"

	# Finally, make some noise to kick the user.
	monitor_bell
}

#
# Pop-up the list of stuff for a length of time.
#
proc appointment_period {duration} {
	global rdb appointment

	set tl ".appt_$duration"
	if [winfo exists $tl] {
		destroy $tl
	}

	toplevel $tl
	appointment_positon_under $tl .
	set title "Appt: $duration"
	wm iconname $tl $title
	wm title $tl $title

	# Fill in the text.
	text $tl.t -wrap word -font $rdb(font)
	set f [open "|$rdb(appointment.${duration}_command)" "r"]
	set h 0
	set w 0
	while {[gets $f l] >= 0} {
		$tl.t insert end "$l\n"
		incr h
		set w [max $w [expr $appointment(text_slop)+[string length $l]]]
	}
	# ignore the return code
	catch {close $f}
	$tl.t conf -state disabled -height $h -width $w
	pack $tl.t -side top -padx $rdb(appointment.padx)

	# Make an "ok button".
	button $tl.b -text "Ok" -command "destroy $tl"
	pack $tl.b -side right -ipadx $rdb(appointment.padx) -padx $rdb(appointment.padx)
}

proc appointment_update_today {} {
	global appointment appt_times appt_data appt_width appt_height env rdb

	set clock [getclock]

	# Check for an appointment.
	if {$appointment(first_appt) < $clock} {
		appointment_signal_appts
	}

	set filetime [file mtime $rdb(appointment.file)]
	set thisday [fmtclock $clock %d]
	if {$filetime == $appointment(lastfiletime) && $thisday == $appointment(lastday)} {
		return
	}

	# File has changed, get the data.
	set f [open "|$rdb(appointment.day_command)" "r"]
	set appointment(today) ""
	.appointment.m delete 0 last
	while {[gets $f l] >= 0} {
		lappend appointment(today) $l
		.appointment.m add command -label $l -font $rdb(font)
	}
	# ignore the return code
	catch {close $f}
	.appointment.m add separator
	foreach i "day week month" {
		.appointment.m add command -label $i -command "appointment_period $i"
	}

	# clear the old variables
	if [info exists appt_times] {
		foreach i $appt_times {
			unset appt_data($i)
			unset appt_height($i)
			unset appt_width($i)
		}
	}
	set appt_times ""

	# Also get the formatted data.
	# Sigh.  We re-do generation of the code because wishx
	# doesn't allow good control over fork and file descriptors
	# like perl.  :-<
	set f [open "| $rdb(appointment.day_command) | $rdb(moduledir)/diary_to_times" r]
	set appt_time 999999999
	while {[gets $f l] >= 0} {
		if [string match ">*" $l] {
			if {![info exists appt_data($appt_time)]} {
				set appt_data($appt_time) ""
				set appt_height($appt_time) 1
				set appt_width($appt_time) 0
			}
			set appt_data($appt_time) "$appt_data($appt_time)\n[csubstr $l 1 end]"
			incr appt_height($appt_time)
			set appt_width($appt_time) \
				[max $appt_width($appt_time) \
				 [expr $appointment(text_slop)+[string length $l]]]
		} else {
			set appt_time $l
			lappend appt_times $l
			if {$l < $appointment(first_appt) && ![info exists appts_shown($l)]} {
				set appointment(first_appt) $l
			}
		}
	}
	close $f

	# remember our work
	set appointment(lastfiletime) $filetime
	set appointment(lastday) [fmtclock [getclock] %d]
}


proc appointment_update {} {
	global monitor_module
	if {$monitor_module(appointment)} {
		appointment_update_today
		after 59000 appointment_update
	}
}

proc appointment_start {} {
	global rdb appointment env
	set appointment(today) {}
	set appointment(lasttime) {}
	set appointment(lastfiletime) {}
	set appointment(first_appt) 999999999
	set appointment(text_slop) 5
	foreach duration {day week month} {
		set_rdb_default appointment.${duration}_command "emacs -batch -l $env(HOME)/LIB/EMACS/calendar-stuff.el -f diary-this-$duration-to-stdout 2>/dev/null"
	}
	set_rdb_default appointment.file "$env(HOME)/LIB/calendar"
	set_rdb_default appointment.padx 0.1i
	set_rdb_default appointment.row 1
	menubutton .appointment -text A -font $rdb(font) -fg $rdb(fg) -bg $rdb(bg) -menu .appointment.m
	mk_rows $rdb(appointment.row)
	menu .appointment.m
	pack .appointment -side $rdb(side) -in ".row$rdb(appointment.row)"
	appointment_update
}

proc appointment_stop {} {
	destroy .appointment
}

