#!/usr/dist/bin/wishx -f

#
# battery.tcl
# Copyright (C) 1995 by John Heidemann
# This program is released under the Gnu Public License, version 2.
# Comments to <johnh@isi.edu>.
#

proc battery_alert {pct threshold} {
	global rdb
	set tl ".battery_alert"
	if [winfo exists $tl] {
		# do nothing
	} else {
		# create it
		toplevel $tl
		wm geometry $tl +100+100
		wm iconname $tl "monitor--battery alert"
		wm title $tl "monitor--battery alert"
		wm transient $tl
		frame $tl.f
		label $tl.f.i -bitmap warning
		pack $tl.f.i -side left -padx 5 -pady 5
		# label
		label $tl.f.l -font $rdb(font)
		pack $tl.f.l -side left
		pack $tl.f -side top
		# ok button
		button $tl.b -text "Ok" -command battery_unalert
		pack $tl.b -side bottom
	}
	$tl.f.l configure -text "Warning:  battery at $pct%,\nbelow threshold $threshold%.\n"
}

proc battery_unalert {} {
	set tl ".battery_alert"
	if [winfo exists $tl] {
		destroy $tl
	}
}

proc battery_update {} {
	global rdb monitor_module
	if {! $monitor_module(battery)} {
		return
	}
	set f [open "/proc/apm" r]
	set tm "b:x"
	set type ""
	set units ""
	set pct ""
	while {[gets $f l] >= 0} {
		if [regexp {^[.0-9]+ [.0-9]+ [x0-9a-fA-F]+ ([x0-9a-fA-F]+) [x0-9a-fA-F]+ [x0-9a-fA-F]+ ([0-9]+)% ([-0-9]+) ([?a-z]+)} $l x ac_status pct time time_units] {
			if {$ac_status == "0x01"} {
				set type "ac"
			} else {
				set type "b"
			}
			if {$rdb(battery.units) == "percent"} {
				set units ":${pct}%"
			} else {
				set units ":${time} ${time_units}"
			}
		}
		if [string match "AC: on *" $l] {
			set type "ac"
		}
		if [string match "AC: off *" $l] {
			set type "b"
		}
		if {$rdb(battery.units) == "percent" && [regexp {Battery life: ([0-9]+)%} $l x pct] == 1} {
			set units ":${pct}%"
		}
		if {$rdb(battery.units) == "time" && [regexp {Battery life: ([-0-9]+) minutes} $l x min] == 1} {
			set units ":${min}m"
		}
	}
	close $f
	set t "$type$units"
	if {$rdb(battery.quiet) && $t=="ac:100%"} { set t "" }
	.battery config -text $t
	# check for low battery
	if {$pct <= $rdb(battery.low_threshold) && $pct < $rdb(battery.recent_low_pct)} {
		# pop up alert
		set rdb(battery.recent_low_pct) $pct
		battery_alert $pct $rdb(battery.low_threshold)
	}
	if {$pct >= $rdb(battery.reset_threshold) && $rdb(battery.recent_low_pct) != 100} {
		set rdb(battery.recent_low_pct) 100
		battery_unalert
	}
	# almost every two minutes
	after 119000 battery_update
}

proc battery_start {} {
	global rdb
	set_rdb_default battery.units percent
	set_rdb_default battery.quiet 0
	set_rdb_default_boolean battery.quiet 0
	set_rdb_default battery.row 1
	set_rdb_default battery.low_threshold 10
	set_rdb_default battery.reset_threshold 25
	set_rdb_default battery.recent_low_pct 100
	mk_rows $rdb(battery.row)
	label .battery -text "battery" -font $rdb(font) -fg $rdb(fg) -bg $rdb(bg)
	pack .battery -side $rdb(side) -in ".row$rdb(battery.row)"
	battery_update
}

proc battery_stop {} {
	destroy .battery
}

